@@ -366,7 +366,7 @@
 #'
 #' Andrew D. Martin, Kevin M. Quinn, and Jong Hee Park. 2011.  ``MCMCpack:
 #' Markov Chain Monte Carlo in R.'', \emph{Journal of Statistical Software}.
-#' 42(9): 1-21.  \url{http://www.jstatsoft.org/v42/i09/}.
+#' 42(9): 1-21.  \doi{10.18637/jss.v042.i09}.
 #'
 #' Daniel Pemstein, Kevin M. Quinn, and Andrew D. Martin.  2007.  \emph{Scythe
 #' Statistical Library 1.0.} \url{http://scythe.lsa.umich.edu}.
@@ -623,6 +623,10 @@
                                  title="MCMCmnl Posterior Sample")
 
     }
+    else{
+        cat("\n\nmcmc.method not equal to one of 'RWM', 'IndMH', or 'slice'.\n")
+        stop("Please respecifify and call MCMCmnl() again.\n")
+    }
 
     return(output)
 
