@@ -1,15 +1,17 @@
 //////////////////////////////////////////////////////////////////////////
-//HMMmultivariateGaussian.cc is C++ code to estimate a Gaussian panel model with a structural break
+// HMMmultivariateGaussian.cc is C++ code to estimate a Gaussian panel model with a structural break
 // y_{it} = \x'_{it}\b + \varepsilon_{it}
 // \varepsilon_{it} \sim \normdist{0}{\sigma^2}
 // Parameters = {beta, sigma2, P_i}// static + changing
 // 
-// Written by Jong Hee Park
-// 11/19/2008
-// Modified by JHP
-// 07/04/2011
+// Jong Hee Park
+// Department of Political Science and International Relations
+// Seoul National University
+// jongheepark@snu.ac.kr
+
+// Written 11/19/2008
+// Modified 07/04/2011
 //
-// Copyright (C) 2008-present Jong Hee Park
 //////////////////////////////////////////////////////////////////////////
 #ifndef HMMMULTIVARIATEGAUSSIAN_CC
 #define HMMMULTIVARIATEGAUSSIAN_CC
@@ -49,12 +51,12 @@ static double ln_invgamma(double theta, double a, double b) {
 
 template <typename RNGTYPE>
 void MultivariateGaussian_impl(rng<RNGTYPE>& stream, 
-			       unsigned int nsubj, unsigned int ntime, unsigned int nobs, 
+			        int nsubj,  int ntime,  int nobs, 
 			       const Matrix<int>& subjectid, const Matrix<int>& timeid, 
 			       const Matrix<>& Y, const Matrix<>& X,  
 			       const Matrix<>& YT, const Matrix<>& XT, 
-			       unsigned int burnin, unsigned int  mcmc, 
-			       unsigned int thin, unsigned int  verbose,  
+			        int burnin,  int  mcmc, 
+			        int thin,  int  verbose,  
 			       double sigma2start,
 			       const Matrix<>& b0, const Matrix<>& B0, 
 			       const double c0, const double d0, 
@@ -62,19 +64,19 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
 			       const Matrix<>& subject_groupinfo, 
 			       Matrix<>& beta_store, Matrix<>& sigma_store, 
 			       double& logmarglike, double& loglike, 
-			       unsigned int chib)
+			        int chib)
 { // redefine constants
   const int K = X.cols();; // ncol(X)
   const int NOBS = nobs;
   const Matrix<> B0inv = invpd(B0);
 
-  const unsigned int tot_iter = burnin + mcmc;
-  const unsigned int nstore = mcmc / thin; // number of draws to store
+  const  int tot_iter = burnin + mcmc;
+  const  int nstore = mcmc / thin; // number of draws to store
   
   // generate posk_arr and post_arr
   // Number of observations by group k
   int *nobsk = new int[nsubj];
-  for (unsigned int k=0; k<nsubj; k++) {
+  for (int k=0; k<nsubj; k++) {
     nobsk[k]=0;
     for (int n=0; n<NOBS; n++) {
       if (subjectid[n]==k+1) {
@@ -85,7 +87,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
 
   // Position of each group in the data-set
   int **posk_arr = new int*[nsubj];
-  for (unsigned int k=0; k<nsubj; k++) {
+  for (int k=0; k<nsubj; k++) {
     posk_arr[k] = new int[nobsk[k]];
     int repk=0;
     for (int n=0; n<NOBS; n++) {
@@ -99,7 +101,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
   // Small fixed matrices indexed on k for data access
   Matrix<double> *Yk_arr = new Matrix<double>[nsubj];
   Matrix<double> *Xk_arr = new Matrix<double>[nsubj];
-  for(unsigned int k=0; k<nsubj; k++) {
+  for(int k=0; k<nsubj; k++) {
     Xk_arr[k] = Matrix<double>(nobsk[k], K);
     Yk_arr[k] = Matrix<double>(nobsk[k], 1);
     for (int l=0; l<nobsk[k]; l++) {
@@ -112,7 +114,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
  
   Matrix<double> *cpXk_arr = new Matrix<double>[nsubj];
   Matrix<double> *tXYk_arr = new Matrix<double>[nsubj];
-  for(unsigned int k=0; k<nsubj; k++) {
+  for(int k=0; k<nsubj; k++) {
     cpXk_arr[k] = crossprod(Xk_arr[k]);
     tXYk_arr[k] = t(Xk_arr[k])*Yk_arr[k];
   }
@@ -131,13 +133,13 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
   /////////////////////////////////////////////////
   // initialize Yhat for the first loop only
   /////////////////////////////////////////////////
-  for (unsigned int iter=0; iter < tot_iter; ++iter){
+  for ( int iter=0; iter < tot_iter; ++iter){
     //////////////////////
     // Step 1. Sample beta 
     //////////////////////
     Matrix<> XVX(K, K);
     Matrix<> XVY(K, 1);
-    for(unsigned int s = 0; s<nsubj; ++s) {       
+    for( int s = 0; s<nsubj; ++s) {       
       XVX = XVX + cpXk_arr[s];
       XVY = XVY + tXYk_arr[s];       
     }
@@ -151,7 +153,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
     /////////////////////////////////////////////////
     double SSE = 0;   
     int counter = 0;
-    for(unsigned int s=0;s<nsubj; ++s){
+    for( int s=0;s<nsubj; ++s){
       int ntime_s = subject_groupinfo(s, 2);
       Matrix<> e = t(Yk_arr[s]-Xk_arr[s]*beta)*(Yk_arr[s] - Xk_arr[s]*beta);
       SSE = SSE + e[0];
@@ -198,10 +200,10 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
     //////////////////////////////////////////////////////////////////
     // 1. pdf.beta | sigma2_g
     //////////////////////////////////////////////////////////////////
-    for (unsigned int iter = 0; iter<nstore; ++iter){
+    for ( int iter = 0; iter<nstore; ++iter){
       Matrix<> XVX(K, K);
       Matrix<> XVY(K, 1);
-      for(unsigned int s = 0; s<nsubj; ++s) {       
+      for( int s = 0; s<nsubj; ++s) {       
 	XVX = XVX + cpXk_arr[s];
 	XVY = XVY + tXYk_arr[s];       
       }
@@ -222,10 +224,9 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
     // 2. pdf.sigma2
     ////////////////////////////////////////////////////////////////// 
     Matrix<> density_sigma2(nstore, 1);  
-    for (unsigned int iter = 0; iter<nstore; ++iter){
+    for ( int iter = 0; iter<nstore; ++iter){
       double SSE = 0;   
-      for(unsigned int s=0;s<nsubj; ++s){
-	int ntime_s = subject_groupinfo(s, 2);
+      for( int s=0;s<nsubj; ++s){
 	Matrix<> e = t(Yk_arr[s]-Xk_arr[s]*beta_st)*(Yk_arr[s] - Xk_arr[s]*beta_st);
 	SSE = SSE + e[0];
       }
@@ -240,7 +241,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
     // likelihood f(y|beta_st, D_st, sigma2_st, P_st)
     //////////////////////////////////////////////////////////////////  
     loglike = 0;
-    for(unsigned int s = 0; s<nsubj; ++s) {       
+    for( int s = 0; s<nsubj; ++s) {       
       int ntime_s = subject_groupinfo(s, 2);     
       Matrix<> Sigma = sigma2_st*eye(ntime_s);
       Matrix<> Mu = Xk_arr[s]*beta_st;
@@ -273,7 +274,7 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
   }
   
   delete[] nobsk;
-  for(unsigned int k=0; k<nsubj; k++) {
+  for( int k=0; k<nsubj; k++) {
     delete[] posk_arr[k];
   }
   delete [] posk_arr;
@@ -287,13 +288,13 @@ void MultivariateGaussian_impl(rng<RNGTYPE>& stream,
 
 template <typename RNGTYPE>
 void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream, 
-				  unsigned int nsubj, unsigned int ntime, 
-				  unsigned int m,  unsigned int nobs, 
+				   int nsubj,  int ntime, 
+				   int m,   int nobs, 
 				  const Matrix<int>& subjectid, const Matrix<int>& timeid, 
 				  const Matrix<>& Y, const Matrix<>& X,  
 				  const Matrix<>& YT, const Matrix<>& XT, 
-				  unsigned int burnin, unsigned int  mcmc, 
-				  unsigned int thin, unsigned int  verbose,  
+				   int burnin,  int  mcmc, 
+				   int thin,  int  verbose,  
 				  Matrix<>& betastart, double sigma2start,
 				  const Matrix<>& b0, const Matrix<>& B0, 
 				  const double c0, const double d0, 
@@ -303,20 +304,20 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
 				  Matrix<>& beta_store, Matrix<>& sigma_store, 
 				  Matrix<>& P_store,  Matrix<>& ps_store, Matrix<>& s_store, 
 				  double& logmarglike, double& loglike, 
-				  unsigned int chib)
+				   int chib)
 { // redefine constants
   const int K = X.cols();; // ncol(X)
   const int NOBS = nobs;
   const Matrix<> B0inv = invpd(B0);
 
-  const unsigned int tot_iter = burnin + mcmc;
-  const unsigned int nstore = mcmc / thin; // number of draws to store
+  const  int tot_iter = burnin + mcmc;
+  const  int nstore = mcmc / thin; // number of draws to store
   const int ns = m + 1; 
   
   // generate posk_arr and post_arr
   // Number of observations by group k
   int *nobsk = new int[nsubj];
-  for (unsigned int k=0; k<nsubj; k++) {
+  for (int k=0; k<nsubj; k++) {
     nobsk[k]=0;
     for (int n=0; n<NOBS; n++) {
       if (subjectid[n]==k+1) {
@@ -327,7 +328,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
 
   // Position of each group in the data-set
   int **posk_arr = new int*[nsubj];
-  for (unsigned int k=0; k<nsubj; k++) {
+  for (int k=0; k<nsubj; k++) {
     posk_arr[k] = new int[nobsk[k]];
     int repk=0;
     for (int n=0; n<NOBS; n++) {
@@ -353,7 +354,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   } 
   // number of observations by time t
   int *nobst = new int[ntime];
-  for (unsigned int k=0; k<ntime; k++) {
+  for (int k=0; k<ntime; k++) {
     nobst[k]=0;
     for (int n=0; n<NOBS; n++) {
       if (timeid[n]==k + 1) {
@@ -363,7 +364,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   }
   // Position of each group in the data-set
   int **post_arr = new int*[ntime];
-  for (unsigned int k=0; k<ntime; k++) {
+  for (int k=0; k<ntime; k++) {
     post_arr[k] = new int[nobst[k]];
     int repk=0;
     for (int n=0; n<NOBS; n++) {
@@ -377,7 +378,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   // XTarr is data transformed for multivariate TS analsysi
   Matrix<>* Xt_arr = new Matrix<>[ntime];
   Matrix<>* Yt_arr = new Matrix<>[ntime];
-  for(unsigned int k=0; k<ntime; k++) {
+  for(int k=0; k<ntime; k++) {
     if (nobst[k] > 0){
       Xt_arr[k] = Matrix<double>(nobst[k], K);
       Yt_arr[k] = Matrix<double>(nobst[k], 1);
@@ -393,7 +394,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   Matrix<double> *cpXt_arr = new Matrix<double>[ntime];
   Matrix<double> *tXt_arr = new Matrix<double>[ntime];
   Matrix<double> *tXYt_arr = new Matrix<double>[ntime];
-  for(unsigned int k=0; k<ntime; k++) {
+  for(int k=0; k<ntime; k++) {
     cpXt_arr[k] = crossprod(Xt_arr[k]);
     tXt_arr[k] = t(Xt_arr[k]);
     tXYt_arr[k] = t(Xt_arr[k])*Yt_arr[k];
@@ -414,7 +415,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   /////////////////////////////////////////////////
   // initialize Yhat for the first loop only
   /////////////////////////////////////////////////
-  for (unsigned int iter=0; iter < tot_iter; ++iter){
+  for ( int iter=0; iter < tot_iter; ++iter){
     //////////////////////
     // Step 1. Sample state
     //////////////////////  
@@ -423,7 +424,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     pr1[0] = 1;
     Matrix<> py(ns, 1);
     Matrix<> pstyt1(ns, 1);
-    for (unsigned int tt=0; tt<ntime ; ++tt) {
+    for ( int tt=0; tt<ntime ; ++tt) {
       if(nobst[tt]>0){
 	int nsubj_s = time_groupinfo(tt, 2);
 	for (int j=0; j<ns;++j){
@@ -478,7 +479,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     Matrix<int> nstate(ns, 1); 
     
     for (int j = 0; j<ns; ++j){
-      for (unsigned int i = 0; i<ntime; ++i){
+      for ( int i = 0; i<ntime; ++i){
 	if (state[i] == j + 1) { 
 	  nstate[j] = nstate[j] + 1;
 	}// end of if
@@ -507,7 +508,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     Matrix<> SSE(ns, 1);
     for (int j = 0; j<ns; ++j){
       beta_count = beta_count + nstate[j];        
-      for(unsigned int s = 0; s<nsubj; ++ s) {    
+      for( int s = 0; s<nsubj; ++ s) {    
 	Matrix<> yj = Yk_arr[s]((beta_count - nstate[j]), 0, (beta_count - 1), 0);
 	Matrix<> Xj = Xk_arr[s]((beta_count - nstate[j]), 0, (beta_count - 1), K-1);  	
 	Matrix<> e = ::t(yj - Xj*::t(beta(j,_)))*(yj - Xj*::t(beta(j,_)));
@@ -524,7 +525,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     //////////////////////   
     double shape1 = 0;
     double shape2 = 0;    
-    for (unsigned int j =0; j<m; ++j){
+    for ( int j =0; j<m; ++j){
       shape1 =  std::abs(P0(j,j) + nstate[j] - 1);
       shape2 =  P0(j,j+1) + 1; //       
       P(j,j) = stream.rbeta(shape1, shape2);
@@ -547,7 +548,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
 	P_store(sampcount,j)= P[j];
       }
       s_store(sampcount,_) = state;
-      for (unsigned int l=0; l<ntime ; ++l){           
+      for ( int l=0; l<ntime ; ++l){           
 	ps_store(l,_) = ps_store(l,_) + ps(l,_);          
       }
       ++sampcount;
@@ -603,11 +604,11 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     //////////////////////////////////////////////////////////////////
     // 1. pdf.beta | sigma2_g, P_g
     //////////////////////////////////////////////////////////////////
-    for (unsigned int iter = 0; iter<nstore; ++iter){
+    for ( int iter = 0; iter<nstore; ++iter){
       int beta_count = 0;
       Matrix<int> nstate(ns, 1);    
       for (int j = 0; j<ns; ++j){
-	for (unsigned int i = 0; i<ntime; ++i){
+	for ( int i = 0; i<ntime; ++i){
 	  if (s_store(iter, i) == j + 1) { 
 	    nstate[j] = nstate[j] + 1;
 	  }// end of if
@@ -638,14 +639,14 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     // 2. pdf.sigma2
      ////////////////////////////////////////////////////////////////// 
     Matrix<> density_sigma2(nstore, ns);  
-    for (unsigned int iter = 0; iter<nstore; ++iter){
+    for ( int iter = 0; iter<nstore; ++iter){
       Matrix<> F(ntime, ns);
       Matrix<> pr1(ns, 1);
       pr1[0] = 1;
       Matrix<> py(ns, 1);
       Matrix<> pstyt1(ns, 1);
       
-      for (unsigned int tt=0; tt<ntime ; ++tt) {
+      for ( int tt=0; tt<ntime ; ++tt) {
 	int nsubj_s = time_groupinfo(tt, 2);
 	for (int j=0; j<ns;++j){
 	  Matrix<> Sigma = eye(nsubj_s)*sigma2[j];
@@ -693,7 +694,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       /////////////////////////////////////////////////
       Matrix<int> nstate(ns, 1); 
       for (int j = 0; j<ns; ++j){
-	for (unsigned int i = 0; i<ntime; ++i){
+	for ( int i = 0; i<ntime; ++i){
 	  if (state(i) == j + 1) { 
 	    nstate[j] = nstate[j] + 1;
 	  }// end of if
@@ -705,7 +706,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       Matrix<int> YN(ns, 1); 
       for (int j = 0; j<ns; ++j){
 	beta_count = beta_count + nstate[j];      
-	for(unsigned int s = 0; s<nsubj; ++ s) {    
+	for( int s = 0; s<nsubj; ++ s) {    
 	  Matrix<> yj = Yk_arr[s]((beta_count - nstate[j]), 0, (beta_count - 1), 0);
 	  Matrix<> Xj = Xk_arr[s]((beta_count - nstate[j]), 0, (beta_count - 1), K-1);  
 	  YN[j] = YN[j] + yj.rows();
@@ -724,7 +725,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       /////////////////////////////////////////////////
       double shape1 = 0;
       double shape2 = 0;    
-      for (unsigned int j =0; j<m; ++j){
+      for ( int j =0; j<m; ++j){
 	shape1 =  std::abs(P0(j,j) + nstate[j] - 1);
 	shape2 =  P0(j,j+1) + 1; //       
 	P(j,j) = stream.rbeta(shape1, shape2);
@@ -740,13 +741,13 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     Matrix<> density_P(nstore, ns);  
     //  Matrix<> density_local_P(ns, 1);  
     
-    for (unsigned int iter = 0; iter < nstore; ++iter){     
+    for ( int iter = 0; iter < nstore; ++iter){     
       Matrix<> F(ntime, ns);
       Matrix<> pr1(ns, 1);
       pr1[0] = 1;
       Matrix<> py(ns, 1);
       Matrix<> pstyt1(ns, 1);     
-      for (unsigned int tt=0; tt<ntime ; ++tt) {
+      for ( int tt=0; tt<ntime ; ++tt) {
 	if(nobst[tt]>0){
 	  int nsubj_s = time_groupinfo(tt, 2);
 	  for (int j=0; j<ns;++j){
@@ -794,7 +795,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       
       Matrix<int> nstate(ns, 1); 
       for (int j = 0; j<ns; ++j){
-	for (unsigned int i = 0; i<ntime; ++i){
+	for ( int i = 0; i<ntime; ++i){
 	  if (state(i) == j + 1) { 
 	    nstate[j] = nstate[j] + 1;
 	  }// end of if
@@ -802,7 +803,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       }
       double shape1 = 0;
       double shape2 = 0;    
-      for (unsigned int j =0; j<m; ++j){
+      for ( int j =0; j<m; ++j){
 	shape1 =  std::abs(P0(j,j) + nstate[j] - 1);
 	shape2 =  P0(j,j+1) + 1; //       
 	P(j,j) = stream.rbeta(shape1, shape2);
@@ -824,9 +825,8 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
     pr1[0] = 1;
     Matrix<> py(ns, 1);
     Matrix<> pstyt1(ns, 1);
-    for (unsigned int tt=0; tt<ntime ; ++tt) {
+    for ( int tt=0; tt<ntime ; ++tt) {
       for (int j=0; j<ns;++j){
-	int nsubj_s = time_groupinfo(tt, 2);
 	Matrix<> Sigma = eye(nsubj)*sigma2_st[j];
 	Matrix<> Mu = Xt_arr[tt]*::t(beta_st(j,_));
 	py[j]  =  ::exp(lndmvn(Yt_arr[tt], Mu, Sigma));
@@ -862,7 +862,7 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
       }
       density_sigma2_prior[j] = ln_invgamma(sigma2_st[j], c0/2, d0/2);
     }
-    for (unsigned int j=0; j<m ; ++j){
+    for ( int j=0; j<m ; ++j){
       density_P_prior[j] = log(dbeta(P_st(j,j), P0(j,j), P0(j,j+1))); 
     }        
     
@@ -882,11 +882,11 @@ void HMMmultivariateGaussian_impl(rng<RNGTYPE>& stream,
   }// end of marginal likelihood computation
   
   delete[] nobst;
-  for(unsigned int k=0; k<ntime; k++) {
+  for(int k=0; k<ntime; k++) {
     delete[] post_arr[k];
   }
   delete[] nobsk;
-  for(unsigned int k=0; k<nsubj; k++) {
+  for(int k=0; k<nsubj; k++) {
     delete[] posk_arr[k];
   }
   delete [] posk_arr;
@@ -932,8 +932,8 @@ extern "C" {
     Matrix<> subject_groupinfo(*nsubj, 3, subject_groupinfodata);
     Matrix<> time_groupinfo(*ntime, 3, time_groupinfodata);
     const int ns = *m + 1;   
-    double logmarglike;
-    double loglike;
+    double logmarglike = 0.0;
+    double loglike = 0.0;
     
     if (*m == 0) {
       Matrix<> beta_store(*betarow, *betacol);
